Package: EMMA
Title: EMMA: Enrichment Methods Matter for enabling fully reproducible
        and provenance-aware pathway analysis
Version: 0.99.5
Authors@R: 
    c(
        person(
            given = "Najla", family = "Abassi", role = c("aut", "cre"), 
            email = "abassi.nejla96@gmail.com",
            comment = c(ORCID = "0000-0001-8357-0938")
        ),
        person(
            given = "Annekathrin", family = "Nedwed", role = c("aut"),
            email = "anneludt@uni-mainz.de",
            comment = c(ORCID = "0000-0002-2475-4945")
        ),
        person(
            given = "Federico", family = "Marini", role = c("aut"), 
            email = "marinif@uni-mainz.de",
            comment = c(ORCID = "0000-0003-3252-7758")
        )
    )
Description: EMMA is a package that provides a provenance-aware execution
    framework for functional enrichment analyses. It allows users to run
    enrichment analyses using existing tools and workflows while explicitly
    capturing analytical parameters during runtime, and returns the native
    enrichment results output together with structured metadata.
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
Imports: AnnotationDbi, cli
Suggests: knitr, rmarkdown, BiocStyle, macrophage, DESeq2, apeglm,
        clusterProfiler, gprofiler2, org.Hs.eg.db, GO.db, mosdef,
        topGO, renv, DeeDeeExperiment, testthat (>= 3.0.0)
VignetteBuilder: knitr
URL: https://github.com/imbeimainz/EMMA
BugReports: https://github.com/imbeimainz/EMMA/issues
biocViews: Software, Pathways, GO, KEGG, GeneSetEnrichment,
        ImmunoOncology, Transcriptomics, SingleCell, GeneExpression,
        DifferentialExpression
Config/testthat/edition: 3
Config/roxygen2/version: 8.1.0
git_url: https://git.bioconductor.org/packages/EMMA
git_branch: devel
git_last_commit: aed0475
git_last_commit_date: 2026-08-12
Repository: Bioconductor 3.24
Date/Publication: 2026-08-13
NeedsCompilation: no
Packaged: 2026-08-13 21:40:04 UTC; biocbuild
Author: Najla Abassi [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8357-0938>),
  Annekathrin Nedwed [aut] (ORCID:
    <https://orcid.org/0000-0002-2475-4945>),
  Federico Marini [aut] (ORCID: <https://orcid.org/0000-0003-3252-7758>)
Maintainer: Najla Abassi <abassi.nejla96@gmail.com>
Depends: R (>= 4.1.0)
