#' EMMA
#'
#' EMMA stands for Enrichment Methods Matter.
#' And EMMA stands to help you in realizing it.
#'
#' @importFrom AnnotationDbi metadata
#' @importFrom utils packageVersion sessionInfo
#' @importFrom cli cli_alert_info
#' @name EMMA-pkg
#' @docType package
"_PACKAGE"
NULL
